% --- Атомарные знания ---
person(p1, 60_000).
vehicle(v1, motorbike, 60_000).
bag(b1, 60_000).
% Позция объекта в определенный момент времени (x, y, z). Присутствует для каждого объекта, у которого
% также есть volumetricPosition.
position(any_object, point(x, y, z), 60_000).
% Позиция, задаваемая в виде многоугольника. Многоугольник представлен набором точек, лежащих на поверхности, 
volumetricPosition(obj, [point(x1, y1, z1), point(x2, y2, z2)], 60_000).
% Двумерное расположение объекта в виде обрамляющей рамка.
bounds(any_obj, box(x1, y1, x2, y2), 60_000).
started(sit_down(p1), 61_000).
started(search(p1, p2), 63_000).
finished(sit_down(p1), 62_000).
finished(search(p1, p2), 64_000).
% Устанавливается один раз - используемый временной шаг
timeStep(1000).
% Время кадра, информация о котором присутствует в базе знаний
timeSlice(60_000).
% Описываем все точки, относящиеся к данному положению в метрах. Гарантируется, что координаты x, y в position
% будут соответствовать - будут заадваться точно для пикселя.
areaLocation(topLeftRoadArea, x, y).
direction(komendanskiyProsperctEast, topLeftRoadArea, vx, vy).

% --- Utility ---

% В случае, если MaxTime - переменная, предикат вернет наибольшее время в базе знаний.
% Опирается на гарантию того, что новые знания зыписываются в начало.
maxTimeSlice(MaxTime) :-
	timeSlice(MaxTime),
	!.

toMillis(dur(Dur, millis), DurMillis) :-
	number(Dur),
	DurMillis is Dur.
toMillis(dur(Dur, sec), DurMillis) :-
	number(Dur),
	DurMillis is Dur * 1000.
toMillis(dur(Dur, min), DurMillis) :-
	number(Dur),
	DurMillis is Dur * 60_000.
toMillis(DurMillis, DurMillis) :-
	number(DurMillis).

% Преобразование расстояния в миллиметры
toMm(dist(Dist, mm), DistMm) :-
	number(Dist),
	DistMm is Dist.
toMm(dist(Dist, m), DistMm) :-
	number(Dist),
	DistMm is Dist * 1000.
toMm(DistMm, DistMm) :-
	number(DistMm).

incTime(BaseTime, Dur, ResultTime) :-
	toMillis(Dur, DurMs),
	ResultTime is BaseTime + DurMs.

decTime(BaseTime, Dur, ResultTime) :-
	toMillis(Dur, DurMs),
	ResultTime is BaseTime - DurMs.

% Итерирует от MinIter до MaxIter включительно с шагом Step.
iter(MinIter, _Step, MaxIter, _Iter) :-
	MinIter > MaxIter,
	!, fail.
iter(MinIter, _Step, _MaxIter, MinIter).
iter(MinIter, Step, MaxIter, Iter) :-
	NextMinIter is MinIter + Step,
	iter(NextMinIter, Step, MaxIter, Iter).

% Задает время, большее StartTime, но не более чем на MaxTimeInt
timeGreater(MinTime, MaxTimeDifference, GreaterTime) :-
	timeSlice(MinTime),
	timeStep(TimeStep),
	toMillis(MaxTimeDifference, MaxTimeDifferenceMillis),
	iter(TimeStep, TimeStep, MaxTimeDifferenceMillis, TimeInt),
	GreaterTime is MinTime + TimeInt.

timeSmaller(MaxTime, MaxTimeDiff, SmallerTime) :-
	timeSlice(MaxTime),
	timeStep(TimeStep),
	toMillis(MaxTimeDiff, MaxTimeDifferenceMillis),
	iter(TimeStep, TimeStep, MaxTimeDifferenceMillis, TimeInt),
	SmallerTime is MaxTime - TimeInt,
	SmallerTime >= 0.

timeBetween(StartTime, FinishTime, Time) :-
	timeStep(TimeStep),
	timeSlice(StartTime),
	timeSlice(FinishTime),
	StartTime =< FinishTime,
	MaxTimeInt is FinishTime - StartTime,
	iter(0, TimeStep, MaxTimeInt, TimeInt),
	Time is StartTime + TimeInt.

% Гарантируется, что между started и finished не вклинится лишний started или finished. А также, в один
% момент времени действие не может закончиться и сразу начаться, но может начаться и сразу закончиться (в этом
% случае StarTime и FinishTime будут равны). Считаем, что утверждение выполнялось от StartTime до FinishTime
% (включительно оба времени).
holds(Statement, StartTime, FinishTime) :-
	started(Statement, StartTime),
	finished(Statement, FinishTime),
	StartTime =< FinishTime,
	\+ (finished(Statement, IntermedTime), StartTime < IntermedTime, IntermedTime < FinishTime).
holds(Statement, StartTime, FinishTime) :-
	started(Statement, StartTime),
	\+ (finished(Statement, FinishTime), StartTime =< FinishTime),
	maxTimeSlice(FinishTime).
holdsAt(Statement, Time) :-
	holds(Statement, StartTime, FinishTime),
	timeBetween(StartTime, FinishTime, Time).

% Предикаты для удобной работы с временем
holdsAt(person(Pers), Time) :-
	person(Pers, Time).

holdsAt(bag(Bag), Time) :-
	bag(Bag, Time).

holdsAt(vehicle(Veh, Type), Time) :-
	vehicle(Veh, Type, Time).

holdsAt(position(Obj, P), Time) :-
	position(Obj, P, Time).

% Предикаты для работы с точками и векторами
vectorOf(point(X1, Y1, Z1), point(X2, Y2, Z2), vector(VX, VY, VZ)) :-
	VX is X2-X1,
	VY is Y2-Y1,
	VZ is Z2-Z1.

vectorLength(vector(X, Y, Z), Length) :-
	Length is sqrt(X**2 + Y**2 + Z**2).

distanceBetweenPoints(P1, P2, Dist) :-
	vectorOf(P1, P2, V),
	vectorLength(V, Dist).

scalarMult(vector(X1, Y1, Z1), vector(X2, Y2, Z2), Scal) :-
	Scal is X1*X2 + Y1*Y2 + Z1*Z2.

cosBetween(V1, V2, Cos) :-
	scalarMult(V1, V2, Scal),
	vectorLength(V1, L1),
	vectorLength(V2, L2),
	Cos is Scal / (L1 * L2).

vectorMult(vector(A1, A2, A3), vector(B1, B2, B3), vector(S1, S2, S3)) :-
	S1 is A2*B3 - A3*B2,
	S2 is A3*B1 - A1*B3,
	S3 is A1*B2 - A2*B1.

pointBetween(point(X1, Y1, Z1), point(X2, Y2, Z2), point(X3, Y3, Z3)) :-
	X3 is (X1 + X2) / 2,
	Y3 is (Y1 + Y2) / 2,
	Z3 is (Z1 + Z2) / 2.

% Возвращает ближайшее время, меньшее Time, когда выполнялось указанное утверждение. Ожидается, что Statement и Time будут заданы.
% Оптимизирует поиск, предполагая, что утверждение выполнялось не более минуты назад.
holdsAtNearestPrevTime(Statement, Time, PrevTime) :-
	timeSmaller(Time, dur(1, min), PrevTime),
	holdsAt(Statement, PrevTime),
	% Поскольку новые события записываются в начало, останавливаемся на первом подходящем моменте времени.
	!.
holdsAtNearestPrevTime(Statement, Time, PrevTime) :-
	% Ожидает, чтоболее свежие знания записываются в начало
	holdsAt(Statement, PrevTime),
	PrevTime < Time,
	!.

% --- Предикаты, описывающие сцену ---

% Средство передвижения подъехало к человеку (сместилось и находится на расстоянии не более 15 м от человека).
% При этом не обязательна остановка (злоумышленник мог высадить подельник на ходу).
% Правило универсально и для машины, и для скутера.
droveUpTo(Veh, Pers, StartTime, FinishTime) :-
	vehicle(Veh, _, FinishTime),
	person(Pers, FinishTime),
	distanceNotGreater(Pers, Veh, dist(15, meter), FinishTime),
	moved(Veh, dist(1, meter), dur(1, min), StartTime, FinishTime),
	\+ (timeBetween(StartTime, FinishTime, Time), onVehic(Veh, Pers, Time)).

% Средство передвижения отъехало от человека (находилось на расстоянии не более 15 м от человека и уехало).
droveAway(Veh, Pers, StartTime, FinishTime) :-
	vehicle(Veh, _, StartTime),
	person(Pers, StartTime),
	distanceNotGreater(Pers, Veh, dist(15, meter), StartTime),
	moved(Veh, dist(1, meter), dur(1, min), StartTime, FinishTime),
	\+ (timeBetween(StartTime, FinishTime, Time), onVehic(Veh, Pers, Time)).

% Obj переместился не менее чем на MinDist за не более чем MaxDur от времени StartTime до FinishTime.
moved(Ojb, MinDist, MaxDur, StartTime, FinishTime) :-
	toMm(MinDist, MinDistMm),
	timeSlice(FinishTime),
	movedMinInterval(Ojb, MinDistMm, MaxDur, StartTime, FinishTime).

% Ожидает, что FinishTime задан. Находит максимальный StartTime, подходящее под критерии moved.
movedMinInterval(Ojb, MinDistMm, MaxDur, StartTime, FinishTime) :-
	position(Ojb, P2, FinishTime),
	timeSmaller(FinishTime, MaxDur, StartTime),
	position(Ojb, P1, StartTime),
	distanceBetweenPoints(P1, P2, Dist),
	Dist >= MinDistMm,
	!.

% Проверяет, что расстояние между двумя точками не больше заданного ограничения.
distanceNotGreater(Obj1, Obj2, MaxDist, Time) :-
	toMm(MaxDist, MaxDistMm),
	distanceBetween(Obj1, Obj2, Time, Dist), 
	Dist =< MaxDistMm.

% Проверяет, что расстояние между двумя точками больше заданного ограничения.
distanceGreater(Obj1, Obj2, MinDist, Time) :-
	toMm(MinDist, MinDistMm),
	distanceBetween(Obj1, Obj2, Time, Dist),
	Dist > MinDistMm.

distanceNotGreaterHappensBetween(Obj1, Obj2, MaxDistance, StartTime, FinishTime) :-
	timeBetween(StartTime, FinishTime, Time),
    distanceNotGreater(Obj1, Obj2, MaxDistance, Time),
    !.

distanceBetween(Obj1, Obj2, Time, Dist) :-
	(person(Obj2, Time) ; animal(Obj2, _, Time)),
	position(Obj1, P1, Time),
	position(Obj2, P2, Time),
	Obj1 \= Obj2,
	distanceBetweenPoints(P1, P2, Dist).
distanceBetween(Obj1, Obj2, Time, Dist) :-
	(vehicle(Obj2, _, Time) ; window(Obj2, Time) ; door(Obj2, Time) ; fence(Obj2, Time) ; wall(Obj2, Time)),
	position(Obj1, P, Time),
	volumetricPosition(Obj2, [FirstP | TailP], Time),
	Obj1 \= Obj2,
	minPointToLineDistance(P, [FirstP | TailP], FirstP, 1_000_000_000, Dist).

minPointToLineDistance(P, [P1, P2 | TP], FirstP, MinDistAcc, MinDist) :-
	pointToLineDistance(P, P1, P2, Dist),
	NewMinDist is min(MinDistAcc, Dist),
	minPointToLineDistance(P, [P2 | TP], FirstP, NewMinDist, MinDist).
minPointToLineDistance(P, [LastP], FirstP, MinDistAcc, MinDist) :-
	pointToLineDistance(P, LastP, FirstP, Dist),
	MinDist is min(MinDistAcc, Dist).

pointToLineDistance(P, LP1, LP2, Dist) :-
	distanceBetweenPoints(P, LP1, AB),
	distanceBetweenPoints(P, LP2, AC),
	(AB < AC -> ClosestP = LP1, FarthestP = LP2 ; ClosestP = LP2, FarthestP = LP1),
	vectorOf(ClosestP, P, V1),
	vectorOf(ClosestP, FarthestP, V2),
	scalarMult(V1, V2, Scal),
	(Scal < 0 -> distanceBetweenPoints(P, ClosestP, Dist) ;
		distanceBetweenPoints(LP1, LP2, BC), vectorOf(P, LP1, ABVec),
		vectorOf(P, LP2, ACVec), cosBetween(ABVec, ACVec, Cos),
		Dist is AB * AC * sin(acos(Cos)) / BC).

% Человек покинул средство предвижения (мотоцикл, машину и т.п.).
% Для мотоцикла - если человек находился "на" мотоцикле.
% Считаем, что действие моментальное. Time - первый момент, когда стало ясно, что человек больше не
% на средстве передвижения.
leftAVehicle(Veh, Pers, LiveTime) :-
	(vehicle(Veh, motorbike, LiveTime); vehicle(Veh, bicycle, LiveTime)),
	person(Pers, LiveTime),
	\+ onVehic(Veh, Pers, LiveTime),
	holdsAtNearestPrevTime(person(Pers), LiveTime, PrevTime),
	onVehic(Veh, Pers, PrevTime).

% Считаем, что человек вышел из машины, если человек появился возле машины, в течение прошлых 10 секунд человек отсутствовал
% на видео, и в течение 5 секунд машина не ехала (это не проезжающая мимо машина).
leftAVehicle(Veh, Pers, LiveTime) :-
	vehicle(Veh, car, LiveTime),
	person(Pers, LiveTime),
	distanceNotGreater(Pers, Veh, dist(1.5, m), LiveTime),
	timeStep(TimeStep),
	CheckFinishTime is LiveTime - TimeStep, 
	decTime(CheckFinishTime, dur(10, sec), CheckStartTime),
	timeSlice(CheckStartTime),
	\+ (timeBetween(CheckStartTime, CheckFinishTime, CheckTime),
		person(Pers, CheckTime)),
	vehicleNotMoving(Veh, LiveTime).

% Считаем, что средство передвижения не двигается (возможно очень медленное движение), если в течение последних 5 секунд
% оно сместилось не более чем на 1 метр.
vehicleNotMoving(Veh, FinishTime) :-
	vehicle(Veh, _, FinishTime),
	position(Veh, P1, FinishTime),
	decTime(FinishTime, dur(5, sec), BaseStartTime),
	holdsAtNearestPrevTime(position(Veh, P2), BaseStartTime, _StartTime),
	distanceBetweenPoints(P1, P2, Dist),
	Dist < 1000.

% Человек сел на/в средство предвижения (мотоцикл, машину и т.п.).
jointAVehicle(Veh, Pers, JoinTime) :-
	(vehicle(Veh, motorbike, JoinTime); vehicle(Veh, bicycle, JoinTime)),
	person(Pers, JoinTime),
	onVehic(Veh, Pers, JoinTime),
	holdsAtNearestPrevTime(person(Pers), JoinTime, PrevTime),
	\+ onVehic(Veh, Pers, PrevTime).

% Считаем, что человек сел в машину, если человек стоял рядом с машиной, затем человек пропал из виду и в течение последующих
% 10 секунд отсутствовал на видео, и в течение 5 секунд до этого события машина не ехала.
jointAVehicle(Veh, Pers, JoinTime) :-
	vehicle(Veh, car, JoinTime),
	timeStep(TimeStep),
	PrevTime is JoinTime - TimeStep,
	person(Pers, PrevTime),
	distanceNotGreater(Pers, Veh, dist(1.5, m), PrevTime),
	CheckStartTime is JoinTime,
	incTime(CheckStartTime, dur(10, sec), CheckFinishTime),
	timeSlice(CheckFinishTime),
	\+ (timeBetween(CheckStartTime, CheckFinishTime, CheckTime),
		person(Pers, CheckTime)),
	vehicleNotMoving(Veh, JoinTime).

% Считается, что человек в определенный момент находится на средстве передвижения, если это мотоцикл,
% велосипед и т. п., а также если положение 2d обрамляющей рамки человека и средства передвижения
% соотносятся определенным образом.
onVehic(Veh, Pers, Time) :-
	(vehicle(Veh, motorbike, Time); vehicle(Veh, bicycle, Time)),
	bounds(Veh, BoxVeh, Time),
	person(Pers, Time),
	bounds(Pers, BoxPers, Time),
	boxIntersection(BoxVeh, BoxPers, BoxInt),
	area(BoxVeh, VehArea),
	area(BoxPers, PersArea),
	area(BoxInt, IntersecArea),
	IntersecArea/PersArea > 0.4,
	IntersecArea/VehArea > 0.3.

% Возвращает обрамляющую рамку, полученную пересечением двух других. Если рамки не пересекаются, тогда либо XInt1 будет больше XInt2,
% либо YInt1 будет больше YInt2, либо оба.
boxIntersection(box(Xf1, Yf1, Xf2, Yf2), box(Xs1, Ys1, Xs2, Ys2), box(XInt1, YInt1, XInt2, YInt2)) :-
	XInt1 is max(Xf1, Xs1),
	YInt1 is max(Yf1, Ys1),
	XInt2 is min(Xf2, Xs2),
	YInt2 is min(Yf2, Ys2).

% Вычисляет площадь обрамляющей рамки. X1, Y1, X2, Y2 должны быть заданы.
area(box(X1, Y1, X2, Y2), Area) :-
	X1 < X2,
	Y1 < Y2,
	Area is (X2 - X1) * (Y2 - Y1),
	!.
area(box(_, _, _, _), 0). 

% Считаем, что Actor1 подошел к Actor2, если начальное расстояние между людьми было более 2м, а стало не более 1.5м, 
% и бОльшую часть пути прошел Actor1. Либо если начальное расстояния было не более 2м, при этом считаем, что
% FinishTime равно StartTime.
% Следует всегда задавать StartTime, поскольку иначе предикат может просто подобрать start time таким образом, что
% начальное и конечное расстояния будут менее 2м.
approached(Actor1, Actor2, MaxDur, StartTime, FinishTime) :- 
	% Начальное время должно быть задано, иначе данный предикат будет подбирать начальное время, чтобы расстояние между
	% людьми было мало.
	number(StartTime),
	distanceGreater(Actor1, Actor2, dist(2, m), StartTime),
	distanceNotGreaterFixedStartTime(Actor1, Actor2, MaxDur, dist(1.5, m), StartTime, FinishTime),
	distance(Actor1, StartTime, FinishTime, Dist1),
	distance(Actor2, StartTime, FinishTime, Dist2),
	Dist1 > Dist2,
	!.
approached(Actor1, Actor2, _MaxDur, StartTime, StartTime) :- 
	number(StartTime),
	distanceNotGreater(Actor1, Actor2, dist(2, m), StartTime).

distanceNotGreaterFixedStartTime(Actor1, Actor2, MaxInt, MaxDist, StartTime, FinishTime) :-
	number(StartTime),
	timeGreater(StartTime, MaxInt, FinishTime),
	distanceNotGreater(Actor1, Actor2, MaxDist, FinishTime),
	!.

% Расстояние, которое прошел Actor за указанный период времени.
distance(Actor, StartTime, FinishTime, Dist) :-
	position(Actor, _, StartTime),
	position(Actor, _, FinishTime),
	StartTime =< FinishTime,
	distanceAcc(Actor, StartTime, FinishTime, Dist).

distanceAcc(Actor, StartTime, FinishTime, DistAcc) :-
	StartTime < FinishTime,
	holdsAtNearestPrevTime(position(Actor, P1), FinishTime, PrevFinishTime),
	position(Actor, P2, FinishTime),
	distanceAcc(Actor, StartTime, PrevFinishTime, PrevDistAcc),
	distanceBetweenPoints(P1, P2, Dist),
	% Тут погрешность учитывать не следует, так как нам нужно выбрать одного из двух людей.
	DistAcc is PrevDistAcc + Dist,
	!.
distanceAcc(_Actor, _StartTime, _FinishTime, 0).

% Считаем, что Pers1 взял вещь у Pers2 в случае, если произошло действие передачи предмета, либо сумка перешла
% от Pers2 во владение Pers1.
tookAnItem(Pers1, Pers2, StartTime, FinishTime) :-
	(holds(transferObject(Pers1, Pers2), StartTime, FinishTime) ;
	possesses(Pers2, Bag, StartTime),
	possesses(Pers1, Bag, FinishTime),
	StartTime < FinishTime,
	Pers1 \= Pers2,
	\+ (posesses(_, Bag, Time), StartTime < Time, Time < FinishTime)).

% Считаем, что человек владеет сумкой, если обрамляющая рамка сумки имеет пересечение только с рамкой этого человека.
% Если же пересечение есть более чем с одним человеком, считаем, что владелец среди них тот, кто владел сумкой последним.
% Если же последний владелец не имеет пересечения с сумкой, считаем, что текущим владельцем является человек с наибольшим
% пересечением.
possesses(Pers, Bag, Time) :-
	bag(Bag, Time),
	bounds(Bag, BoxBag, Time),
	intersectPerson(BoxBag, Pers, _, Time),
	\+ (intersectPerson(BoxBag, AnotherPers, _, Time), AnotherPers \= Pers),
	!.
possesses(Pers, Bag, Time) :-
	bag(Bag, Time),
	bounds(Bag, BoxBag, Time),
	intersectPerson(BoxBag, Pers, _, Time),
	holdsAtNearestPrevTime(bag(Bag), Time, PrevTime),
	possesses(Pers, Bag, PrevTime),
	!.
possesses(Pers, Bag, Time) :-
	bag(Bag, Time),
	bounds(Bag, BoxBag, Time),
	intersectPerson(BoxBag, Pers, IntersectAreaPers, Time),
	\+ (intersectPerson(BoxBag, AnotherPers, IntersectAreaAnotherPers, Time),
		AnotherPers \= Pers,
		IntersectAreaAnotherPers > IntersectAreaPers),
	!.

intersectPerson(Box, Pers, IntersectArea, Time) :-
	person(Pers, Time),
	bounds(Pers, BoxPers, Time),
	boxIntersection(Box, BoxPers, BoxInt),
	area(BoxInt, IntersectArea),
	IntersectArea > 0.

% DeviantBehaviour
robbingWithVehicle(Vict, Robb, Veh, StartTime, FinishTime) :-
	droveUpTo(Veh, Vict, StartTime, ArriveTime),
	leftAVehicle(Veh, Robb, ArriveTime),
	approached(Robb, Vict, dur(30, sec), ArriveTime, ApproachTime),
	(tookAnItem(Robb, Vict, ActStartTime, ActFinishTime) ;
		holds(searchedPockets(Robb, Vict), ActStartTime, ActFinishTime) ;
		holds(tugBetween(Robb, Vict), ActStartTime, ActFinishTime)),
	ApproachTime =< ActStartTime,
	incTime(ActFinishTime, dur(1, min), RetreatTimeLimit),
	jointAVehicle(Veh, Robb, RetreatTime),
	ActFinishTime =< RetreatTime,
	RetreatTime =< RetreatTimeLimit,
	droveAway(Veh, Vict, RetreatTime, FinishTime).

% Определяет направление, в котором объект покинул сцену (человек, транспорт и др.). Для одной камеры может быть задано
% множество направлений, которые определяются положением на кадре и вектором. Положения нескольких направлений могут пересекаться.
% Считается, что объект покинул сцену в определенном напрвлении, если на данный момент объект отсутствует на сцене,
% в последний момент присутствия объект находился в области этого направления, и вектор движения объекта соответствовал
% вектору этого направления (скалярное произведение векторов было больше 0). В случае, если объект находился в области
% нескольких пересекающихся направлений, выбирается то направление, с которым скалярное произведение векторов наибольшее.
% Ожидается, что Obj будет задан, а LeavingTime - нет.
leavingDirection(Obj, presents, LeavingTime) :-
	maxTimeSlice(CurrentTime),
	position(Obj, _, CurrentTime),
	LeavingTime is CurrentTime,
	!.
leavingDirection(Obj, Direction, LeavingTime) :-
	maxTimeSlice(CurrentTime),
	holdsAtNearestPrevTime(position(Obj, point(X, Y, _Z)), CurrentTime, LeavingTime),
	movementVector(Obj, LeavingTime, vector(MVx, MVy, _MVz)),
	directionForMovement(Direction, X, Y, MVx, MVy, DirectCoorientation),
    \+ (directionForMovement(AnotherDirection, X, Y, MVx, MVy, AnotherDirectCoorientation),
    AnotherDirection \= Direction,
    AnotherDirectCoorientation > DirectCoorientation),
    !.

directionForMovement(Direction, X, Y, Vx, Vy, Coorientation) :-
	areaLocation(AreaLocation, X, Y),
    direction(Direction, AreaLocation, DVx, DVy),
    cosBetween(vector(Vx, Vy, 0), vector(DVx, DVy, 0), Coorientation),
    Coorientation > 0.

% Определяет направление движения объекта в определенный момент. В качестве направления движения принимается вектор
% смещения между двумя точками во времени. Выбирается ближайшая по времени начальная точка таким образом, чтобы размер
% вектора был не менее 1 метра. Если смещение объекта за последнюю минуту меньше 1 метра, то считается, что движения нет.
movementVector(Obj, FinishTime, V) :-
	position(Obj, P2, FinishTime),
	timeSmaller(FinishTime, dur(1, min), StartTime),
	position(Obj, P1, StartTime),
	vectorOf(P1, P2, V),
	vectorLength(V, Dist),
	Dist >= 1000,
	!.

